@@ -25,66 +25,51 @@ from collections import defaultdict
 import re
 import os
 import subprocess
-import tempfile
 import Namcap.package
 from Namcap.ruleclass import *
+from Namcap.util import is_elf
+from Namcap.rules.rpath import get_rpaths
+
+from elftools.elf.elffile import ELFFile
+from elftools.elf.dynamic import DynamicSection
 
 libcache = {'i686': {}, 'x86-64': {}}
 
-def figurebitsize(line):
+def scanlibs(fileobj, filename, custom_libs):
 	"""
-	Given a line of output from readelf (usually Shared library:) return
-	'i686' or 'x86-64' if the binary is a 32bit or 64bit binary
-	"""
-
-	address = line.split()[0]
-	if len(address) == 18: # + '0x' + 16 digits
-		return 'x86-64'
-	else:
-		return 'i686'
-
-def scanlibs(fileobj, filename, sharedlibs):
-	"""
-	Run "readelf -d" on a file-like object (e.g. a TarFile)
+	Find shared libraries in a file-like binary object
 
 	If it depends on a library, store that library's path.
 
-	sharedlibs: a dictionary { library => set(ELF files using that library) }
+	returns: a dictionary { library => set(ELF files using that library) }
 	"""
-	shared = re.compile('Shared library: \[(.*)\]')
 
-	# test magic bytes
-	magic = fileobj.read(4)
-	if magic[:4] != b"\x7fELF":
-		return
+	if not is_elf(fileobj):
+		return {}
 
-	# read the rest of file
-	tmp = tempfile.NamedTemporaryFile(delete=False)
-	tmp.write(magic + fileobj.read())
-	tmp.close()
-
-	try:
-		p = subprocess.Popen(["readelf", "-d", tmp.name],
-				env = {"LANG": "C"},
-				stdout=subprocess.PIPE,
-				stderr=subprocess.PIPE)
-		var = p.communicate()
-		assert(p.returncode == 0)
-		for j in var[0].decode('ascii').splitlines():
-			n = shared.search(j)
-			# Is this a Shared library: line?
-			if n != None:
-				# Find out its architecture
-				architecture = figurebitsize(j)
-				try:
-					libpath = os.path.abspath(
-							libcache[architecture][n.group(1)])[1:]
-					sharedlibs.setdefault(libpath, set()).add(filename)
-				except KeyError:
-					# We didn't know about the library, so add it for fail later
-					sharedlibs.setdefault(n.group(1), set()).add(filename)
-	finally:
-		os.unlink(tmp.name)
+	elffile = ELFFile(fileobj)
+	sharedlibs = defaultdict(set)
+	for section in elffile.iter_sections():
+		if not isinstance(section, DynamicSection):
+			continue
+		for tag in section.iter_tags():
+			# DT_NEEDED means shared library
+			if tag.entry.d_tag != 'DT_NEEDED':
+				continue
+			bitsize = elffile.elfclass
+			architecture = {32:'i686', 64:'x86-64'}[bitsize]
+			libname = tag.needed
+			if libname in custom_libs:
+				sharedlibs[custom_libs[libname][1:]].add(filename)
+				continue
+			try:
+				libpath = os.path.abspath(
+						libcache[architecture][libname])[1:]
+				sharedlibs[libpath].add(filename)
+			except KeyError:
+				# We didn't know about the library, so add it for fail later
+				sharedlibs[libname].add(filename)
+	return sharedlibs
 
 def finddepends(liblist):
 	"""
@@ -152,12 +137,21 @@ class SharedLibsRule(TarballRule):
 		dependlist = {}
 		filllibcache()
 		os.environ['LC_ALL'] = 'C'
+		pkg_so_files = ['/' + n for n in tar.getnames() if '.so' in n]
 
 		for entry in tar:
 			if not entry.isfile():
 				continue
 			f = tar.extractfile(entry)
-			scanlibs(f, entry.name, liblist)
+			# find anything that could be rpath related
+			rpath_files = {}
+			if is_elf(f):
+				rpaths = list(get_rpaths(f))
+				f.seek(0)
+				for n in pkg_so_files:
+					if any(n.startswith(rp) for rp in rpaths):
+						rpath_files[os.path.basename(n)] = n
+			liblist.update(scanlibs(f, entry.name, rpath_files))
 			f.close()
 
 		# Ldd all the files and find all the link and script dependencies
@@ -181,9 +175,9 @@ class SharedLibsRule(TarballRule):
 
 		# Check for packages in testing
 		for i in dependlist.keys():
-			p = Namcap.package.load_from_db(i, 'testing')
+			p = Namcap.package.load_testing_package(i)
 			q = Namcap.package.load_from_db(i)
-			if p != None and q != None and p["version"] == q["version"] :
+			if p is not None and q is not None and p["version"] == q["version"] :
 				self.warnings.append(("dependency-is-testing-release %s", i))
 
 # vim: set ts=4 sw=4 noet:
